/* 
 *  ==========================================================================
 *  WeatherDataModel.java: Simple data model for storing weather conditions ..
 * 
 *  Written by: Mark Austin                                         April 2022 
 *  ==========================================================================
 */ 

package filter04;

import au.com.bytecode.opencsv.CSVReader;
import au.com.bytecode.opencsv.CSVWriter;

import java.lang.Math;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

import java.util.ArrayList;
import java.util.List;
import java.util.Date;

import java.time.LocalDate;

public class WeatherDataModel {
   String dataFileName;
   boolean DEBUG = true;
   int count = 0;

   float minPrecipitation;
   float maxPrecipitation;
   float minTemp;
   float maxTemp;

   // Arraylist for data items ...

   private static ArrayList<WeatherDataItem> data;

   // Default constructor method ...

   public WeatherDataModel() {
      data = new ArrayList<WeatherDataItem>();
   };

   // ===============================================
   // Load data file into model ...
   // ===============================================

   public void loadDataFile(String dataFileName ) throws IOException {
      this.dataFileName = dataFileName;

      // Open CSVReader and skip over header ...

      CSVReader reader = new CSVReader( new FileReader( dataFileName ));
      String header [] = reader.readNext();

      // Loop over daily weather measurements ...

      String [] nextLine;
      while ((nextLine = reader.readNext()) != null) {
         String date           = nextLine[0];
         Float precipitation  = Float.parseFloat( nextLine[1] );
         Float maxTemperature = Float.parseFloat( nextLine[2] );
         Float minTemperature = Float.parseFloat( nextLine[3] );

         // Store data in Weather data item ...

         WeatherDataItem item = new WeatherDataItem();
         item.setDate ( date );
         item.setPrecipitation ( precipitation );
         item.setMinTemp ( minTemperature );
         item.setMaxTemp ( maxTemperature );

         // Update max, min temps and precipitation ...

         if (count == 0) {
            minPrecipitation = maxPrecipitation = precipitation;
            minTemp = minTemperature;
            maxTemp = maxTemperature;
         } else {
            minPrecipitation = Math.min( minPrecipitation, precipitation);
            maxPrecipitation = Math.max( maxPrecipitation, precipitation);
            minTemp = Math.min( minTemp, minTemperature);
            maxTemp = Math.max( maxTemp, maxTemperature);
         }

         // Add weather item to array list ...

         data.add(item);

         // Print data item ...

         if(DEBUG == true) {
            System.out.println( item.toString() );
         }

         count = count + 1;
      }
   }

   // =================================================
   // Process populated data model ...
   // =================================================

   public void process() {
   }

   // ===================================================
   // Create string description of Weather data model ...
   // ===================================================

   public String toString() {
      StringBuffer buf = new StringBuffer();

      buf.append( String.format("WeatherDataModel ... \n"));
      buf.append( String.format("======================================= ... \n"));
      buf.append( String.format("--- Data file: %s ... \n", dataFileName ));
      buf.append( String.format("--- No records: %d ... \n", count ));
      buf.append( String.format("--- Min precipitation: %4.1f ... \n", minPrecipitation ));
      buf.append( String.format("--- Max precipitation: %4.1f ... \n", maxPrecipitation ));
      buf.append( String.format("--- Min temp: %4.1f ... \n", minTemp ));
      buf.append( String.format("--- Max temp: %4.1f ... \n", maxTemp ));
      buf.append( String.format("======================================= ... \n"));


      return buf.toString();
   }
}
