/**
 * ===========================================================
 * SymbolTableImpl.java: Interface for the symbol table stack.
 *
 * Copyright (c) 2009 by Ronald Mak
 * For instructional purposes only.  No warranties.
 * ===========================================================
 */

package symbol02;

import java.util.ArrayList;

public interface SymbolTableStack {

    /**
     * Setter.
     * @param entry the symbol table entry for the main program identifier.
     */

    public void setProgramId(SymbolTableEntry entry);

    /**
     * Getter.
     * @return the symbol table entry for the main program identifier.
     */

    public SymbolTableEntry getProgramId();

    /**
     * Getter.
     * @return the current nesting level.
     */
    public int getCurrentNestingLevel();

    /**
     * Return the local symbol table which is at the top of the stack.
     * @return the local symbol table.
     */
    public SymbolTable getLocalSymbolTable();

    /**
     * Push a new symbol table onto the stack.
     * @return the pushed symbol table.
     */

    public SymbolTable push();

    /**
     * Push a symbol table onto the stack.
     * @param symTab the symbol table to push.
     * @return the pushed symbol table.
     */

    public SymbolTable push(SymbolTable symTab);

    /**
     * Pop a symbol table off the stack.
     * @return the popped symbol table.
     */

    public SymbolTable pop();

    /**
     * Create and enter a new entry into the local symbol table.
     * @param name the name of the entry.
     * @return the new entry.
     */

    public SymbolTableEntry enterLocal(String name);

    /**
     * Look up an existing symbol table entry in the local symbol table.
     * @param name the name of the entry.
     * @return the entry, or null if it does not exist.
     */

    public SymbolTableEntry lookupLocal(String name);

    /**
     * Look up an existing symbol table entry throughout the stack.
     * @param name the name of the entry.
     * @return the entry, or null if it does not exist.
     */

    public SymbolTableEntry lookup(String name);
}
