/*
 * ======================================================================
 * DimensionsLibrary.java:
 *
 * Written by: Mark Austin
 * ======================================================================
 */

package symbol02.datatypes;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

public class DimensionsLibrary {

   // Create singleton for library of builtin units ...

   public static HashMap<String,Dimensions> library;

   // Constructor methods ...

   public DimensionsLibrary() {
      this.library = new HashMap<String,Dimensions>();
      buildLibrary();
   }

   // Retrieve library hashmap ...

   public HashMap getLibrary() {
      return library;
   }

   // ===============================================================
   // Build library of well-known unit names and their properties ...
   // ===============================================================

   public void buildLibrary() {

      // SI Length units ...

      ArrayList<Dimensions> builtins = new ArrayList<Dimensions>();
      builtins.add ( new Dimensions ("micron", 1e-6,  1,  0,  0, 0, 0, DimensionsType.SI) );
      builtins.add ( new Dimensions (    "mm", 1e-3,  1,  0,  0, 0, 0, DimensionsType.SI) );
      builtins.add ( new Dimensions (    "cm", 1e-2,  1,  0,  0, 0, 0, DimensionsType.SI) );
      builtins.add ( new Dimensions (    "dm", 1e-1,  1,  0,  0, 0, 0, DimensionsType.SI) );
      builtins.add ( new Dimensions (     "m", 1e-0,  1,  0,  0, 0, 0, DimensionsType.SI) );
      builtins.add ( new Dimensions (    "km", 1e+3,  1,  0,  0, 0, 0, DimensionsType.SI) );

      // SI Mass units ...

      builtins.add ( new Dimensions (  "g",   1E-3,  0,  1,  0, 0, 0, DimensionsType.SI ));
      builtins.add ( new Dimensions ( "kg",    1.0,  0,  1,  0, 0, 0, DimensionsType.SI ));
      builtins.add ( new Dimensions ( "Mg",   1E+6,  0,  1,  0, 0, 0, DimensionsType.SI ));

      // SI_US Time units ...

      builtins.add ( new Dimensions ( "sec",    1.0,  0,  0,  1, 0, 0, DimensionsType.SI_US ));
      builtins.add ( new Dimensions (  "ms", 0.0010,  0,  0,  1, 0, 0, DimensionsType.SI_US ));
      builtins.add ( new Dimensions ( "min",   60.0,  0,  0,  1, 0, 0, DimensionsType.SI_US ));
      builtins.add ( new Dimensions (  "hr", 3600.0,  0,  0,  1, 0, 0, DimensionsType.SI_US ));

      // SI Temperature units ...

      builtins.add ( new Dimensions ( "deg_C",  1.0,  0,  0,  0, 1, 0, DimensionsType.SI ));
      builtins.add ( new Dimensions ( "DEG_C",  1.0,  0,  0,  0, 1, 0, DimensionsType.SI ));

      // SI Frequency units ...

      builtins.add ( new Dimensions (    "Hz",  1.0,  0,  0, -1, 0, 0, DimensionsType.SI_US));

      // SI_US Speed units ...

      builtins.add ( new Dimensions ( "rpm", 1.0/60.0,  0,  0, -1, 0, 0, DimensionsType.SI_US ));
      builtins.add ( new Dimensions ( "cps",      1.0,  0,  0, -1, 0, 0, DimensionsType.SI_US ));

      // SI Force units ...

      builtins.add ( new Dimensions (  "N",      1.0,  1,  1, -2, 0, 0, DimensionsType.SI ));
      builtins.add ( new Dimensions ( "kN",     1E+3,  1,  1, -2, 0, 0, DimensionsType.SI ));
      builtins.add ( new Dimensions ( "kgf", 9.80665,  1,  1, -2, 0, 0, DimensionsType.SI )); 

      // SI Pressure units ...

      builtins.add ( new Dimensions (  "Pa",   1.0, -1,  1, -2, 0, 0, DimensionsType.SI ));
      builtins.add ( new Dimensions ( "kPa",  1E+3, -1,  1, -2, 0, 0, DimensionsType.SI ));
      builtins.add ( new Dimensions ( "MPa",  1E+6, -1,  1, -2, 0, 0, DimensionsType.SI ));
      builtins.add ( new Dimensions ( "GPa",  1E+9, -1,  1, -2, 0, 0, DimensionsType.SI ));

      // SI Energy units ...

      builtins.add ( new Dimensions ( "Jou",   1.0,  2,  1, -2, 0, 0, DimensionsType.SI ));
      builtins.add ( new Dimensions (  "kJ",  1E+3,  2,  1, -2, 0, 0, DimensionsType.SI ));

      // SI Power units ....

      builtins.add ( new Dimensions ( "Watt",  1.0,  2,  1, -3, 0, 0, DimensionsType.SI ));
      builtins.add ( new Dimensions (   "kW", 1e+3,  2,  1, -3, 0, 0, DimensionsType.SI ));

      // US Length units ...

      builtins.add ( new Dimensions (       "in",  1.0,  1,  0,  0, 0, 0, DimensionsType.US ));
      builtins.add ( new Dimensions (      "mil", 1E-3,  1,  0,  0, 0, 0, DimensionsType.US ));
      builtins.add ( new Dimensions ( "micro_in", 1E-6,  1,  0,  0, 0, 0, DimensionsType.US ));
      builtins.add ( new Dimensions (       "ft", 12.0,  1,  0,  0, 0, 0, DimensionsType.US ));
      builtins.add ( new Dimensions (     "yard", 36.0,  1,  0,  0, 0, 0, DimensionsType.US ));
      builtins.add ( new Dimensions ( "mile",  63360.0,  1,  0,  0, 0, 0, DimensionsType.US ));

      // US Volume units ...

      builtins.add ( new Dimensions ( "gallon",  1.0,  3,  0,  0, 0, 0, DimensionsType.US ));
      builtins.add ( new Dimensions ( "barrel",  1.0,  3,  0,  0, 0, 0, DimensionsType.US ));

      // US Mass units ...

      builtins.add ( new Dimensions (    "lb",      1.0,  0,  1,  0, 0, 0, DimensionsType.US ));
      builtins.add ( new Dimensions ( "grain", 1.0/7E+3,  0,  1,  0, 0, 0, DimensionsType.US ));
      builtins.add ( new Dimensions (   "ton",     2E+3,  0,  1,  0, 0, 0, DimensionsType.US ));
      builtins.add ( new Dimensions (   "klb",     1E+3,  0,  1,  0, 0, 0, DimensionsType.US ));

      // US Temperature units ...

      builtins.add ( new Dimensions ( "deg_F", 1.0,  0,  0,  0, 1, 0, DimensionsType.US ));
      builtins.add ( new Dimensions ( "DEG_F", 1.0,  0,  0,  0, 1, 0, DimensionsType.US ));

      // US Force units ...

      builtins.add ( new Dimensions (  "lbf", 1e+0,  1,  1, -2, 0, 0, DimensionsType.US ));
      builtins.add ( new Dimensions ( "kips", 1e+3,  1,  1, -2, 0, 0, DimensionsType.US ));

      // US Pressure units ...

      builtins.add ( new Dimensions ( "psi",  1.0, -1,  1, -2, 0, 0, DimensionsType.US ));
      builtins.add ( new Dimensions ( "ksi", 1e+3, -1,  1, -2, 0, 0, DimensionsType.US ));

      // Plane angle....

      builtins.add ( new Dimensions ( "deg", 0.01745329,  0,  0,  0, 0, 1, DimensionsType.SI_US ));
      builtins.add ( new Dimensions ( "rad",        1.0,  0,  0,  0, 0, 1, DimensionsType.SI_US ));

      // Transfer arraylist to hashmap ....

      Iterator walk = builtins.iterator();
      while ( walk.hasNext() != false ) {
          Dimensions d = (Dimensions) walk.next();
          library.put ( d.getName(), d );
      }
   }

   // =========================================================
   // Exercise methods in DimensionsLibrary ....
   // =========================================================

   public static void main( String args [] ) {

      // Part 01: Create and print a simple dimension ....

      System.out.println("");
      System.out.println("*** Exercise methods in aladdin/DimensionsLibrary ...");
      System.out.println("*** ============================================= ...");

      DimensionsLibrary lib = new DimensionsLibrary ();
      System.out.println( lib.getLibrary().toString() );
   }
}
